;--------------------------------------------------------------------------
; snthup4k.h
;
; Upper 4k of snthcart
;
; Includes the following functions that
; are called from the main program:
;
; beatBox
; calcVolume
; readKeyboards
;
; Upper 4k also includes beat data, which is
; in the snthbeat.h file. (included below)
;
;--------------------------------------------------------------------------


	org $2000
	rorg $1000


;--------------------------------------------------------------------------
; Keyboard read Delay
;--------------------------------------------------------------------------
; Put this here to avoid page boundaries
;--------------------------------------------------------------------------
keyDelay
	txa
	ldx #150
keyDelayLoop
	dex
	bne keyDelayLoop
	tax

	rts ; keyDelay




;--------------------------------------------------------------------------
; Beat Data
;--------------------------------------------------------------------------

	include snthbeat.h


;--------------------------------------------------------------------------
; Store Key
;--------------------------------------------------------------------------
; Store a pressed key into the key buffer.  This
; function looks for the next available slot 
; and stores the key there.  If all four slots
; are filled, the key is dropped.  
; - temp16 must contain the buffer location
; - ACC must contain the value to be stored
;--------------------------------------------------------------------------
storeKey
	; Get the location of the keys (right before key array)
	ldx temp16L
	dex
	
	; Get previous number of keys
	ldy 0,x
	cpy #4
	beq noStore

	; Store the new key
	sta (temp16L),y

	; Increase it by one and store
	iny
	sty 0,x

noStore

	rts ; storeKey



;--------------------------------------------------------------------------
; Sound Type Array High (for upper 4k bank)
;--------------------------------------------------------------------------
soundTypeArrayHigh
	byte 4,6,7,8
	byte 15,12,1,3



;--------------------------------------------------------------------------
; Extract Beat Data
;--------------------------------------------------------------------------
; Each byte of beat data contains the frequency and 
; sound type data.  This function separates and decodes them.
;
; The encoding is: the 3 high bits contain the encoded sound
; type and the lower 5 bits contain the freq data.
;
; - ACC must contain beat byte
;
; - ACC will return the freq
; - X will return the sound type
;--------------------------------------------------------------------------
extractBeatData
	cmp #255
	beq noNote
	
	tax

	; Extract freq data and push it
	and #%00011111
	pha

	txa
	lsr
	lsr
	lsr
	lsr
	lsr
	tax
	
	lda soundTypeArrayHigh,x

	tax
	pla

	rts

noNote
	lda #255

	rts ; extractBeatData : other rts




;--------------------------------------------------------------------------
; Accent Reader
;--------------------------------------------------------------------------
; Each set of beat data is followed by 4 accept bytes.
; Each bit in order represents the accent (on or off)
; of its corresponding 32nd note.  This function
; returns the attenuation of a note in a beat.
;
; - temp16 must contain an indirect pointer to the beat data
; - Y must contain the current 32nd note
;
; = will return the attenuation in ACC
;--------------------------------------------------------------------------
bitMaskArray
	byte #%10000000
	byte #%01000000
	byte #%00100000
	byte #%00010000
	byte #%00001000
	byte #%00000100
	byte #%00000010
	byte #%00000001

accentReader
	and #%00011111

	; There are 4 accent bytes.  Get the
	; correct one.
	tya
	pha

	and #%00011000
	lsr
	lsr
	lsr

	; The 4 accent bytes are after the 32 bytes
	; that make up the beat.
	clc
	adc #32
	tay

	; Now set up a mask for the bit
	pla
	and #%00000111
	tax
	
	lda (temp16L),y
	and bitMaskArray,x
	beq noAccent

	; Accent, so don't attenuate
	lda #0

	rts

noAccent

	; No accent, so attenuate a bit
	lda #8

	rts ; accentReader



;--------------------------------------------------------------------------
; Calc Beat Offset
;--------------------------------------------------------------------------
; Calculates the beat offset based on the given beat number,
; the current beat bank, and the beat counter (to determine if a fill is needed)
;
; - set ACC to the beat number
;
; = returns offset in X
;--------------------------------------------------------------------------
calcBeatOffset

	asl
	asl
	tay

	; use temp16L since temp is used in beatBox
	sta temp16L

	lda beatBank
	asl
	asl
	asl
	asl
	asl
	asl	; x64
	clc
	adc temp16L
	tax
	
	lda beat
	and #%01100000
	cmp #96
	bne noFill

	inx
	inx

noFill

	rts ; calcBeatOffset







;--------------------------------------------------------------------------
; Beat Box
;--------------------------------------------------------------------------
; Plays up to two pre-programmed beats simlutaneously.
; - Assumes use of right keypad only for beat box
; - ACC must contain the number of voices available (1 or 2)
; 
; If two oscillators are available, the other oscillator
; will be used for a second beat if one is selected.
;--------------------------------------------------------------------------
beatBox

	; Save number of voices
	pha

; Determine selected beats
;--------------------------------------------------------------------------

	lda keypad2
	cmp #255
	beq resetLock

	lda beatLock
	bne endBeat

	lda keypad2
	cmp #11
	bne beatsOn
	


	ldx beatBox1
	cpx #11
	bne noKillBeatReset

	ldx #0
	stx beatResetCount

noKillBeatReset

	sta beatBox2

	; set beat lock
	ldx #255
	stx beatLock

beatsOn
	
	sta beatBox1

	lda keypad2+1
	cmp #255
	beq beat2Off

	sta beatBox2

	; set beat lock
	lda #255
	sta beatLock
	jmp endBeat
	
beat2Off
	lda #11
	sta beatBox2
	jmp endBeat

resetLock

	; clear beat lock
	lda #0
	sta beatLock

endBeat


	
; Play beat 1
;--------------------------------------------------------------------------
	lda beatBox1
	cmp #11
	beq skipBeatReset

	; If a beat is activated, and the arpeggiator
	; and beats haven't been in use for a few seconds,
	; then reset the beat/measure/tempoCounter
	lda beatResetCount
	bne noBeatTempoReset

	; Reset measure/beat
	lda #0
	sta beat
	sta tempoCount

noBeatTempoReset
	; Note that this is decremented in the colorCycle function
	lda #255
	sta beatResetCount

skipBeatReset

	; Get address of selected beat
	lda beatBox1
	jsr calcBeatOffset

	lda beatArray,x
	sta temp16L
	inx
	lda beatArray,x
	sta temp16H

	; The variable, beat, contains the 32nd note
	; that the beat is currently on.
	lda beat
	and #%00011111
	tay

	lda (temp16L),y
	jsr extractBeatData

	sta note2
	stx sound2	

	ldy beat
	jsr accentReader
	clc
	adc balAttRight
	sta atten2

	; disable adsr control for beatbox
	ldx #0
	stx adsrEnable2


; Play beat 2
;--------------------------------------------------------------------------
	tax

	; Get number of voices off stack
	pla

	; If beat1 is playing a rest, then play 
	; beat2 regardless of how many voices there are.
	cpx #255
	beq playBeat2	

	; Get number of voices
	cmp #2
	beq playBeat2

	jmp beatBoxEnd

playBeat2
	; Save #ofVoices
	sta temp

	; Get address of selected beat
	lda beatBox2
	jsr calcBeatOffset

	lda beatArray,x
	sta temp16L
	inx
	lda beatArray,x
	sta temp16H

	; The variable, beat, contains the 32nd note
	; that the beat is currently on.
	lda beat
	and #%00011111
	tay

	lda (temp16L),y
	jsr extractBeatData

	; Get number of voices
	ldy temp
	cpy #2
	beq playBeat2on1

	; If there's only 1 voice, then play
	; beat2 on osc2

	sta note2
	stx sound2

	ldy beat
	jsr accentReader
	clc
	adc balAttRight
	sta atten2

	; disable adsr control for beatbox
	ldx #0
	stx adsrEnable2

	jmp beatBoxEnd

playBeat2on1
	; If both voices are available, play
	; beat2 on osc1

	sta note1
	stx sound1	

	ldy beat
	jsr accentReader
	clc
	adc balAttRight
	sta atten1

	; disable adsr control for beatbox
	ldx #0
	stx adsrEnable1

beatBoxEnd

; If no beat is playing, then enable adsr for long releases
;--------------------------------------------------------------------------
	lda beatLock
	bne endAdsrEnable

	lda beatBox1
	cmp #11
	bne endAdsrEnable

	lda beatBox2
	cmp #11
	bne endAdsrEnable

	lda #255
	sta adsrEnable2

endAdsrEnable

	rts ; beatBox



;--------------------------------------------------------------------------
; Calculate Volume
;--------------------------------------------------------------------------
; This function handles volume.  It
; must be called at some point before
; notePlayer in order to calculate the volume
; of each note.  
; 
; Volume is adjusted:
; - depending on the sound used.  some sounds are too loud.
; - for the tremelo feature
; - when the note=255, the volume is zero
; - for the slow attack/release feature
;--------------------------------------------------------------------------
calcVolume8k

	lda #255
	sta vol1
	sta vol2

	ldy adsrSetting
	cpy #4
	beq asyncTrem
	cpy #8
	beq syncTrem

	jmp endTremelo

; Handle Tremelo
;--------------------------------------------------------------------------
	
asyncTrem
	inc adsrLevel1

	lda adsrLevel1
	cmp #7
	bmi quitVolume

	ldx #4

	ldy adsrEnable1
	beq tremA1
	stx vol1
tremA1
	ldy adsrEnable2
	beq tremA2
	stx vol2
tremA2

	cmp #11
	bmi quitVolume

	lda #0
	sta adsrLevel1
	
	jmp quitVolume

syncTrem
	lda beat
	and #00000001
	beq quitVolume

	ldx #4

	ldy adsrEnable1
	beq tremS1
	stx vol1
tremS1
	ldy adsrEnable2
	beq tremS2
	stx vol2
tremS2


quitVolume
	ldx note1
	ldy note2
	
	cpx #255
	bne note1on

	lda #0
	sta vol1

note1on
	cpy #255
	bne note2on

	lda #0
	sta vol2

note2on

	jmp endVolume

endTremelo


; Attack
;--------------------------------------------------------------------------

	lda note1
	cmp #255
	beq attack2

	tya 
	and #1
	beq regAttack1

	lda adsrEnable1
	beq regAttack1

	lda note1
	cmp lastNote1
	bne resetAttack1

	ldx adsrLevel1
	cpx #15
	beq regAttack1

	lda frame
	and #%00000111
	bne noInc1

	inx
noInc1

	stx vol1
	stx adsrLevel1
	
	jmp attack2

regAttack1
	lda note1
	sta lastNote1

	lda sound1
	sta lastSound1

	ldx #15
	stx vol1
	stx adsrLevel1

	jmp attack2

resetAttack1
	sta lastNote1
	
	lda sound1
	sta lastSound1

	lda #0
	sta adsrLevel1
	sta vol1

attack2

	lda note2
	cmp #255
	beq endAttack

	tya
	and #1
	beq regAttack2

	lda adsrEnable2
	beq regAttack2

	lda note2
	cmp lastNote2
	bne resetAttack2

	ldx adsrLevel2
	cpx #15
	beq regAttack2

	lda frame
	and #%00000111
	bne noInc2
	inx 
noInc2

	stx vol2
	stx adsrLevel2

	jmp endAttack

regAttack2
	lda note2
	sta lastNote2

	lda sound2
	sta lastSound2

	ldx #15
	stx vol2
	stx adsrLevel2

	jmp endAttack

resetAttack2
	sta lastNote2

	lda sound2
	sta lastSound2

	lda #0
	sta adsrLevel2
	sta vol2

endAttack



; Handle release
;--------------------------------------------------------------------------

	lda note1
	cmp #255
	bne rel2

	tya 
	and #2
	beq regRelease1

	ldx adsrEnable1
	beq regRelease1

	ldx adsrLevel1
	beq regRelease1

	lda frame
	and #%00000111
	bne noDex1

	dex 
noDex1

	stx vol1
	stx adsrLevel1
	
	ldx lastNote1
	stx note1
	
	ldx lastSound1
	stx sound1
	
	jmp rel2

regRelease1

	ldx #0
	stx vol1

	ldx #255
	stx lastNote1

rel2

	lda note2
	cmp #255
	bne endVolume

	tya
	and #2
	beq regRelease2

	ldx adsrEnable2
	beq regRelease2

	ldx adsrLevel2
	beq regRelease2

	lda frame
	and #%00000111
	bne noDex2
	dex 
noDex2

	stx vol2
	stx adsrLevel2

	ldx lastNote2
	stx note2

	ldx lastSound2
	stx sound2
	
	jmp endVolume

regRelease2

	ldx #0
	stx vol2

	ldx #255
	stx lastNote2

endVolume


; Clear adsrLevel when adsr is disabled
;--------------------------------------------------------------------------

	lda adsrEnable1
	bne enable2

	lda #0
	sta adsrLevel1

enable2

	lda adsrEnable2
	bne endEnable

	lda #0
	sta adsrLevel2

endEnable

; Handle attenuation
;--------------------------------------------------------------------------
	; Attenuation 1	
	lda vol1
	beq handleAtten2
	sbc atten1	
	bcs notZero1

	lda #1

notZero1
	sta vol1

handleAtten2
	; Attenuation 2
	lda vol2
	beq endAtten
	sbc atten2
	bcs notZero2

	lda #1

notZero2
	sta vol2

endAtten

	lda #0
	sta atten1
	sta atten2

	rts ; calcVolume









;--------------------------------------------------------------------------
; Clear temp Keypad Buffers
;--------------------------------------------------------------------------
; This function is copied in the lower 4k as clearKeys
;--------------------------------------------------------------------------
clearKeys8k
	lda #255
	sta tempKeypad1;
	sta tempKeypad1+1;
	sta tempKeypad1+2;
	sta tempKeypad1+3;
	sta tempKeypad2;
	sta tempKeypad2+1;
	sta tempKeypad2+2;
	sta tempKeypad2+3;
	lda #0
	sta tempKeys1
	sta tempKeys2	
	rts ; clearKeys



;--------------------------------------------------------------------------
; Read Keyboards
;--------------------------------------------------------------------------
readKeyboards8k


; *** Read Keyboards ***

	lda #$FF		; Switch Port A to output
	sta SWACNT

	; Because of the delay required to read 
	; each row of the keypads, this alternates
	; reading rows 1&2 and rows 3&4 each frame.
	inc keyRead
	lda keyRead
	and #%00000001
	beq row12
	jmp row34

; Read Rows 1 and 2
;--------------------------------------------------------------------------
row12
	; clear keypad buffers
	jsr clearKeys8k

	; Set up to read row 1 and then delay
	lda #$EE
	sta SWCHA
	jsr keyDelay

	; Set up address to store keys
	ldx #<tempKeypad1
	ldy #>tempKeypad1
	stx temp16L
	sty temp16H

	bit INPT0		; left keypad top row
	bmi notL1
	lda #0
	jsr storeKey
notL1
	bit INPT1
	bmi notL2
	lda #1
	jsr storeKey
notL2
	bit INPT4
	bmi notL3
	lda #2
	jsr storeKey
notL3

	; Set up address to store keys
	ldx #<tempKeypad2
	ldy #>tempKeypad2
	stx temp16L
	sty temp16H

	bit INPT2		; right keypad top row
	bmi notR1
	lda #0
	jsr storeKey
notR1
	bit INPT3
	bmi notR2
	lda #1
	jsr storeKey
notR2
	bit INPT5
	bmi notR3
	lda #2
	jsr storeKey
notR3

	lda #$DD		; ROW 2
	sta SWCHA
	jsr keyDelay

	; Set up address to store keys
	ldx #<tempKeypad1
	ldy #>tempKeypad1
	stx temp16L
	sty temp16H

	bit INPT0		; left keypad row 2
	bmi notL4
	lda #3
	jsr storeKey
notL4
	bit INPT1
	bmi notL5
	lda #4
	jsr storeKey
notL5
	bit INPT4
	bmi notL6
	lda #5
	jsr storeKey
notL6

	; Set up address to store keys
	ldx #<tempKeypad2
	ldy #>tempKeypad2
	stx temp16L
	sty temp16H

	bit INPT2		; right keypad row 2
	bmi notR4
	lda #3
	jsr storeKey
notR4
	bit INPT3
	bmi notR5
	lda #4
	jsr storeKey
notR5
	bit INPT5
	bmi notR6
	lda #5
	jsr storeKey
notR6
	jmp endKey


; Rows 3 and 4
;--------------------------------------------------------------------------
row34
	lda #$BB		; ROW 3
	sta SWCHA
	jsr keyDelay

	; Set up address to store keys
	ldx #<tempKeypad1
	ldy #>tempKeypad1
	stx temp16L
	sty temp16H

	bit INPT0		; left keypad row 3
	bmi notL7
	lda #6
	jsr storeKey
notL7
	bit INPT1
	bmi notL8
	lda #7
	jsr storeKey
notL8
	bit INPT4
	bmi notL9
	lda #8
	jsr storeKey
notL9

	; Set up address to store keys
	ldx #<tempKeypad2
	ldy #>tempKeypad2
	stx temp16L
	sty temp16H

	bit INPT2		; right keypad row 3
	bmi notR7
	lda #6
	jsr storeKey
notR7
	bit INPT3
	bmi notR8
	lda #7
	jsr storeKey
notR8
	bit INPT5
	bmi notR9
	lda #8
	jsr storeKey
notR9

	lda #$77		; ROW 4
	sta SWCHA
	jsr keyDelay

	; Set up address to store keys
	ldx #<tempKeypad1
	ldy #>tempKeypad1
	stx temp16L
	sty temp16H

	bit INPT0		; left keypad row 4
	bmi notLstar
	lda #9
	jsr storeKey
notLstar
	bit INPT1
	bmi notL0
	lda #10
	jsr storeKey
notL0
	bit INPT4
	bmi notLpound
	lda #11
	jsr storeKey
notLpound

	; Set up address to store keys
	ldx #<tempKeypad2
	ldy #>tempKeypad2
	stx temp16L
	sty temp16H

	bit INPT2		; right keypad row 4
	bmi notRstar
	lda #9
	jsr storeKey
notRstar
	bit INPT3
	bmi notR0
	lda #10
	jsr storeKey
notR0
	bit INPT5
	bmi notRpound
	lda #11
	jsr storeKey
notRpound

; Copy temp keypad array to real keypad array
;--------------------------------------------------------------------------
	ldx #0
keyLoop1
	lda tempKeypad1,x
	sta keypad1,x
	inx
	cpx #4
	bne keyLoop1

	ldx #0
keyLoop2
	lda tempKeypad2,x
	sta keypad2,x
	inx
	cpx #4
	bne keyLoop2

	; Copy variables that keep track of how many keys were pressed
	lda tempKeys1
	sta keys1
	lda tempKeys2
	sta keys2

; This is where the Row 1 and 2 reader skips to...
;--------------------------------------------------------------------------
endKey

	lda #$FF		; Switch portA back to INPUT
	sta SWCHA
	lda #$00
	sta SWACNT

	rts ; readKeyboards



	
;--------------------------------------------------------------------------
; Version information
;--------------------------------------------------------------------------
	org $2F00
	byte 1,1,0,255	; version

;--------------------------------------------------------------------------
; Read Keyboards Volume Caller
;--------------------------------------------------------------------------
; Bankswitching code to call read keyboards subroutine
; and then switch back to bank 1.
;--------------------------------------------------------------------------

	org $2FC3
	rorg $1FC3

	jsr readKeyboards8k
	ldx $1FF8  		;(Switch back to bank 1)        


;--------------------------------------------------------------------------
; Calc Volume Caller
;--------------------------------------------------------------------------
; Bankswitching code to call calc volume subroutine
; and then switch back to bank 1.
;--------------------------------------------------------------------------

	org $2FD3
	rorg $1FD3

	jsr calcVolume8k
	ldx $1FF8  		;(Switch back to bank 1)        


;--------------------------------------------------------------------------
; Beat Player Caller
;--------------------------------------------------------------------------
; Bankswitching code to call beat player subroutine
; and then switch back to bank 1.
;--------------------------------------------------------------------------

	org $2FE3
	rorg $1FE3

	jsr beatBox
	ldx $1FF8  		;(Switch back to bank 1)        


;--------------------------------------------------------------------------
; The cart will start up in this bank.  Make sure it switches
; back to bank 1 on startup.
;--------------------------------------------------------------------------
	org $2FF0
	rorg $1FF0

	lda $1FF8  ; switch to bank 1 (3 bytes)
	;jmp Start  ; (in bank 1)

	org $2FFC	; Program startup vector
	.word $1FF0
	.word $1FF0
	
